--[[
	Copyright (C) 2006-2007 Nymbia

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
]]
local L = AceLibrary("AceLocale-2.2"):new("CombatAnnounce")
CombatAnnounce = AceLibrary("AceAddon-2.0"):new("FuBarPlugin-2.0", "AceDB-2.0", "AceEvent-2.0", "AceConsole-2.0")
CombatAnnounce:RegisterDB("CombatAnnounceDB")
local dewdrop = AceLibrary("Dewdrop-2.0")
local tablet = AceLibrary("Tablet-2.0")

local options, optionsupdated, db

local combatlogevents = {
	"CHAT_MSG_COMBAT_CREATURE_VS_CREATURE_HITS",
	"CHAT_MSG_COMBAT_CREATURE_VS_CREATURE_MISSES",
	"CHAT_MSG_COMBAT_CREATURE_VS_PARTY_HITS",
	"CHAT_MSG_COMBAT_CREATURE_VS_PARTY_MISSES",
	"CHAT_MSG_COMBAT_CREATURE_VS_SELF_HITS",
	"CHAT_MSG_COMBAT_CREATURE_VS_SELF_MISSES",
	"CHAT_MSG_COMBAT_ERROR",
	"CHAT_MSG_COMBAT_FACTION_CHANGE",
	"CHAT_MSG_COMBAT_FRIENDLYPLAYER_HITS",
	"CHAT_MSG_COMBAT_FRIENDLYPLAYER_MISSES",
	"CHAT_MSG_COMBAT_FRIENDLY_DEATH",
	"CHAT_MSG_COMBAT_HONOR_GAIN",
	"CHAT_MSG_COMBAT_HOSTILEPLAYER_HITS",
	"CHAT_MSG_COMBAT_HOSTILEPLAYER_MISSES",
	"CHAT_MSG_COMBAT_HOSTILE_DEATH",
	"CHAT_MSG_COMBAT_MISC_INFO",
	"CHAT_MSG_COMBAT_PARTY_HITS",
	"CHAT_MSG_COMBAT_PARTY_MISSES",
	"CHAT_MSG_COMBAT_PET_HITS",
	"CHAT_MSG_COMBAT_PET_MISSES",
	"CHAT_MSG_COMBAT_SELF_HITS",
	"CHAT_MSG_COMBAT_SELF_MISSES",
	"CHAT_MSG_COMBAT_XP_GAIN",
	"CHAT_MSG_MONSTER_EMOTE",
	"CHAT_MSG_MONSTER_SAY",
	"CHAT_MSG_MONSTER_WHISPER",
	"CHAT_MSG_MONSTER_YELL",
	"CHAT_MSG_RAID_BOSS_EMOTE",
	"CHAT_MSG_SKILL",
	"CHAT_MSG_SPELL_AURA_GONE_OTHER",
	"CHAT_MSG_SPELL_AURA_GONE_SELF",
	"CHAT_MSG_SPELL_BREAK_AURA",
	"CHAT_MSG_SPELL_CREATURE_VS_CREATURE_BUFF",
	"CHAT_MSG_SPELL_CREATURE_VS_CREATURE_DAMAGE",
	"CHAT_MSG_SPELL_CREATURE_VS_PARTY_BUFF",
	"CHAT_MSG_SPELL_CREATURE_VS_PARTY_DAMAGE",
	"CHAT_MSG_SPELL_CREATURE_VS_SELF_BUFF",
	"CHAT_MSG_SPELL_CREATURE_VS_SELF_DAMAGE",
	"CHAT_MSG_SPELL_DAMAGESHIELDS_ON_OTHERS",
	"CHAT_MSG_SPELL_DAMAGESHIELDS_ON_SELF",
	"CHAT_MSG_SPELL_FAILED_LOCALPLAYER",
	"CHAT_MSG_SPELL_FRIENDLYPLAYER_BUFF",
	"CHAT_MSG_SPELL_FRIENDLYPLAYER_DAMAGE",
	"CHAT_MSG_SPELL_HOSTILEPLAYER_BUFF",
	"CHAT_MSG_SPELL_HOSTILEPLAYER_DAMAGE",
	"CHAT_MSG_SPELL_ITEM_ENCHANTMENTS",
	"CHAT_MSG_SPELL_PARTY_BUFF",
	"CHAT_MSG_SPELL_PARTY_DAMAGE",
	"CHAT_MSG_SPELL_PERIODIC_CREATURE_BUFFS",
	"CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE",
	"CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_BUFFS",
	"CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_DAMAGE",
	"CHAT_MSG_SPELL_PERIODIC_PARTY_BUFFS",
	"CHAT_MSG_SPELL_PERIODIC_PARTY_DAMAGE",
	"CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS",
	"CHAT_MSG_SPELL_PERIODIC_SELF_DAMAGE",
	"CHAT_MSG_SPELL_PET_BUFF",
	"CHAT_MSG_SPELL_PET_DAMAGE",
	"CHAT_MSG_SPELL_SELF_BUFF",
	"CHAT_MSG_SPELL_SELF_DAMAGE",
	"CHAT_MSG_SPELL_TRADESKILLS",
}

function CombatAnnounce:OnInitialize()
	self:RegisterChatCommand({"/combatannounce", "/ca"}, options)
	self:RegisterDefaults("profile", {
		triggers = {
			['*'] = {
				searchstr = L['Search Text Here'],
				chatmode = 'PARTY',
			}
		},
		running = true,
	})
	self.hideWithoutStandby = true
	self.hasIcon = true
end
function CombatAnnounce:OnEnable()
	if self.db.profile.running then
		for _, event in ipairs(combatlogevents) do
			self:RegisterEvent(event, 'OnEvent')
		end
	end
	if self:IsIconShown() then
		self:SetIcon("Interface\\Icons\\INV_Misc_Fish_42")
	else
		self:HideIcon()
	end
	optionsupdated()
	self:UpdateText()
end

function CombatAnnounce:OnEvent(msg)
	for _,v in pairs(db) do
		if msg:match(v.searchstr) then
			local mode = v.chatmode
			if mode == 'SELF' then
				self:Print("|cffff9999"..msg)
			elseif mode == 'GROUP' then
				if GetNumRaidMembers() > 0 then
					if GetRealNumRaidMembers() > 0 then
						SendChatMessage(L["(CA): %s"]:format(msg), 'RAID')
					else
						SendChatMessage(L["(CA): %s"]:format(msg), 'BATTLEGROUND')
					end
				elseif GetNumPartyMembers() > 0 then
					SendChatMessage(L["(CA): %s"]:format(msg), 'PARTY')
				else
					self:Print("|cffff9999"..msg)
				end
			else
				SendChatMessage(L["(CA): %s"]:format(msg), v.chatmode)
			end
		end
	end
end
function CombatAnnounce:OnClick()
	if self.db.profile.running then
		self:UnregisterAllEvents()
		self:Print(L['Announcements Disabled'])
		self.db.profile.running = false
	else
		for _, event in ipairs(combatlogevents) do
			self:RegisterEvent(event, 'OnEvent')
		end
		self:Print(L['Announcements Enabled'])
		self.db.profile.running = true
	end
	self:UpdateText()
end
function CombatAnnounce:OnTooltipUpdate()
	tablet:SetHint(L["Right-Click to edit triggers."])
end

function CombatAnnounce:OnMenuRequest()
	dewdrop:FeedAceOptionsTable(options)
end
function CombatAnnounce:OnTextUpdate()
	if self.db.profile.running then
		self:SetText(L["CombatAnnounce"])
	else
		self:SetText("|cff888888"..L["CombatAnnounce"])
	end
end

local channels = {
	'PARTY',
	'GUILD',
	'OFFICER',
	'RAID',
	'RAID_WARNING',
	'SAY',
	'YELL',
	'SELF',
	'GROUP',
	'BATTLEGROUND'
}
local function getsearchstring(n)
	return db[n].searchstr
end
local function setsearchstring(n,v)
	db[n].searchstr = v
	optionsupdated()
end
local function getchannel(n)
	return db[n].chatmode
end
local function setchannel(n,v)
	db[n].chatmode = v
	optionsupdated()
end
local function delete(n)
	db[n] = nil
	optionsupdated()
end
function optionsupdated()
	db = CombatAnnounce.db.profile.triggers
	options.args.triggers.args = {}
	for k,v in pairs(db) do
		options.args.triggers.args[k] = {
			type = 'group',
			name = k,
			desc = k,
			args = {
				searchstr = {
					type = 'text',
					name = L["Search String"],
					desc = L["Edit this event's search string"],
					get = getsearchstring,
					set = setsearchstring,
					usage = L["< lua patterns >"],
					passValue = k,
					order = 1,
				},
				channel = {
					type = 'text',
					name = L["Channel"],
					desc = L["Select channel to announce to"],
					get = getchannel,
					set = setchannel,
					validate = channels,
					passValue = k,
					order = 2,
				},
				delete = {
					type = 'execute',
					name = L["Delete"],
					desc = L["Delete this trigger"],
					func = delete,
					passValue = k,
					order = 3,
				},
			},
		}
	end
end
CombatAnnounce.OnProfileEnable = optionsupdated
options = {
	type = 'group',
	args = {
		triggers = {
			type = 'group',
			name = L["Triggers"],
			desc = L["Edit existing triggers"],
			args = {},
		},
		space = {
			type = 'header',
			order = 101,
		},
		new = {
			name = L["New Trigger"],
			desc = L["Create a new trigger"],
			type = 'text',
			get = false,
			set = function(v)
				db[v] = db[v]
				optionsupdated()
			end,
			usage = L["< Trigger Name >"],
			validate = function(v)
				if rawget(db, v) then
					return false
				else
					return true
				end
			end,
			order = 102,
		},
	},
}